/************************************************************************************
dijkstrairs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***************************************************************************************//
//* TestInput.cpp							     2010.10.19 *//
//*											*//
//* Test Input RTMプログラム [Ver.0.1.0]						*//
//***************************************************************************************//


// -*- C++ -*-
/*!
 * @file  TestInput.cpp
 * @brief Test Input for Dijkstra Module
 * $Date$
 *
 * $Id$
 */

#include "TestInput.h"

// Module specification
// <rtc-template block="module_spec">
static const char* testinput_spec[] =
  {
    "implementation_id", "TestInputGoal",
    "type_name",         "TestInputGoal",
    "description",       "Test Input for Dijkstra Module",
    "version",           "1.0.0",
    "vendor",            "IRS",
    "category",          "Map",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "2",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_x", "238.0",
    "conf.default.02_y", "-80.0",
    "conf.default.03_theta", "0.0",

    ""
  };
// </rtc-template>

TestInputGoal::TestInputGoal(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_PositionOut("Position", m_Position),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("Position", m_PositionOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

TestInputGoal::~TestInputGoal()
{
}


RTC::ReturnCode_t TestInputGoal::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("01_x", m_01_x, "238.0");
	bindParameter("02_y", m_02_y, "-80.0");
	bindParameter("03_theta", m_03_theta, "0.0");

	// </rtc-template>


	//-- [初期化] ---------------------------------------------------------------------------//

	m_Position.id.length(0);									// メモリ確保 [id]
	m_Position.error.length(0);									// メモリ確保 [error]

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}



/*
RTC::ReturnCode_t TestInputGoal::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInputGoal::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInputGoal::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t TestInputGoal::onActivated(RTC::UniqueId ec_id)
{

	//-- [コンフィギュレーション読み込み] ---------------------------------------------------//

	x     = m_01_x;											// データ格納 [x座標]
	y     = m_02_y;											// データ格納 [y座標]
	theta = m_03_theta;										// データ格納 [theta]

	//---------------------------------------------------------------------------------------//


	//-- [デバッグconfiguration] ------------------------------------------------------------//

	#ifndef DEBUG_CONFIGURATION									// デバッグ表示開始

	printf("\n[configuration]\n");									// label
	printf("01_x     = %lf\n", x);									// x
	printf("02_y     = %lf\n", y);									// y
	printf("03_theta = %lf\n\n", theta);								// theta
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//-- [タイムスタンプ格納] ---------------------------------------------------------------//

	timestamp = gettimeofday_sec();									// タイムスタンプ格納
	sec  = (unsigned long int)timestamp;								// タイムスタンプ格納 [sec]
	nsec = (unsigned long int)(fmod(timestamp, 1) * 1000000);					// タイムスタンプ格納 [nsec]

	//---------------------------------------------------------------------------------------//


	//-- [データ出力] -----------------------------------------------------------------------//

	m_Position.tm.sec  = sec;									// タイムスタンプ格納
	m_Position.tm.nsec = nsec;									// タイムスタンプ格納
	m_Position.x       = x;										// x座標格納
	m_Position.y       = y;										// y座標格納
	m_Position.theta   = theta;									// theta格納
	m_PositionOut.write();										// データ出力

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t TestInputGoal::onDeactivated(RTC::UniqueId ec_id)
{

	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t TestInputGoal::onExecute(RTC::UniqueId ec_id)
{

	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t TestInputGoal::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInputGoal::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInputGoal::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInputGoal::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TestInputGoal::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void TestInputGoalInit(RTC::Manager* manager)
  {
    RTC::Properties profile(testinput_spec);
    manager->registerFactory(profile,
                             RTC::Create<TestInputGoal>,
                             RTC::Delete<TestInputGoal>);
  }

};


