/************************************************************************************
dijkstrairs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***************************************************************************************//
//* Dijkstra.cpp							     2010.10.19 *//
//*											*//
//* Dijkstra RTMプログラム [Ver.0.1.0]							*//
//*											*//
//* コンフィギュレーション								*//
//*	01_location		: 地域名						*//
//*	02_mapset		: 利用者マップセット名					*//
//*	03_type			: データタイプ						*//
//*	04_originx		: 対象原点x座標 [mm]					*//
//*	05_originy		: 対象原点y座標 [mm]					*//
//*	06_sizex		: 対象範囲x座標 [mm]					*//
//*	07_sizey		: 対象範囲y座標 [mm]					*//
//*	08_pixelsizex		: ピクセルサイズx座標 [mm]				*//
//*	09_pixelsizey		: ピクセルサイズy座標 [mm]				*//
//*	10_gui			: GUI動作						*//
//*				  [off, on]						*//
//*	11_guiaddress		: GUIプログラムアドレス					*//
//*	12_ip			: IPアドレス						*//
//*	13_sendport		: 送信ポート番号					*//
//*	14_receiveport		: 受信ポート番号					*//
//***************************************************************************************//


// -*- C++ -*-
/*!
 * @file  Dijkstra.cpp
 * @brief Path Planning
 * $Date$
 *
 * $Id$
 */

#include "Dijkstra.h"

//-- [変数宣言] ---------------------------------------------------------------------------------//
int wait_flag = 1;											// フラグ格納変数 [読み取り用]
//-----------------------------------------------------------------------------------------------//

// Module specification
// <rtc-template block="module_spec">
static const char* dijkstra_spec[] =
  {
    "implementation_id", "Dijkstra",
    "type_name",         "Dijkstra",
    "description",       "Path Planning",
    "version",           "1.0.0",
    "vendor",            "IRS",
    "category",          "Map",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_location", "kobe_green_space_park",
    "conf.default.02_mapset", "irs",
    "conf.default.03_type", "hazard",
    "conf.default.04_originx", "100000",
    "conf.default.05_originy", "-100000",
    "conf.default.06_sizex", "170000",
    "conf.default.07_sizey", "80000",
    "conf.default.08_pixelsizex", "1000",
    "conf.default.09_pixelsizey", "1000",
    "conf.default.10_gui", "on",
    "conf.default.11_guiaddress", "../gui/dijkstra",
    "conf.default.12_ip", "127.0.0.1",
    "conf.default.13_sendport", "10008",
    "conf.default.14_receiveport", "10007",

    ""
  };
// </rtc-template>

Dijkstra::Dijkstra(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_RobotPositionIn("RobotPosition", m_RobotPosition),
    m_GoalPositionIn("GoalPosition", m_GoalPosition),
    m_MeshDataIn("MeshData", m_MeshData),
    m_PathOut("Path", m_Path),
    m_MeshOrderOut("MeshOrder", m_MeshOrder),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("RobotPosition", m_RobotPositionIn);
  registerInPort("GoalPosition", m_GoalPositionIn);
  registerInPort("MeshData", m_MeshDataIn);
  
  // Set OutPort buffer
  registerOutPort("Path", m_PathOut);
  registerOutPort("MeshOrder", m_MeshOrderOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

Dijkstra::~Dijkstra()
{
}


RTC::ReturnCode_t Dijkstra::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("01_location", m_01_location, "kobe_green_space_park");
	bindParameter("02_mapset", m_02_mapset, "irs");
	bindParameter("03_type", m_03_type, "hazard");
	bindParameter("04_originx", m_04_originx, "100000");
	bindParameter("05_originy", m_05_originy, "-100000");
	bindParameter("06_sizex", m_06_sizex, "170000");
	bindParameter("07_sizey", m_07_sizey, "80000");
	bindParameter("08_pixelsizex", m_08_pixelsizex, "1000");
	bindParameter("09_pixelsizey", m_09_pixelsizey, "1000");
	bindParameter("10_gui", m_10_gui, "on");
	bindParameter("11_guiaddress", m_11_guiaddress, "../gui/dijkstra");
	bindParameter("12_ip", m_12_ip, "127.0.0.1");
	bindParameter("13_sendport", m_13_sendport, "10008");
	bindParameter("14_receiveport", m_14_receiveport, "10007");

	// </rtc-template>


	//-- [初期化] ---------------------------------------------------------------------------//

	m_MeshOrder.location.length(DATASIZE);								// メモリ確保 [location]
	m_MeshOrder.mapset.length(DATASIZE);								// メモリ確保 [mapset]
	m_MeshOrder.type.length(DATASIZE);								// メモリ確保 [type]

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t Dijkstra::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Dijkstra::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Dijkstra::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t Dijkstra::onActivated(RTC::UniqueId ec_id)
{

	//-- [初期化] ---------------------------------------------------------------------------//

	gui_flag       = 0;										// GUI動作フラグ初期化

	sec            = 0;										// タイムスタンプ初期化 [sec]
	nsec           = 0;										// タイムスタンプ初期化 [nsec]
	robot_x        = 0;										// ロボットx座標初期化
	robot_y        = 0;										// ロボットy座標初期化
	robot_theta    = 0.0;										// ロボットtheta初期化

	sec_2          = 0;										// タイムスタンプ初期化 [sec]
	nsec_2         = 0;										// タイムスタンプ初期化 [nsec]
	goal_x         = 0;										// ゴールx座標初期化
	goal_y         = 0;										// ゴールy座標初期化

	sec_3          = 0;										// タイムスタンプ初期化 [sec]
	nsec_3         = 0;										// タイムスタンプ初期化 [nsec]
	mode_3         = 0;										// アクセスモード初期化
	strcpy(location_3, "");										// 地域名初期化
	strcpy(mapset_3, "");										// 利用者マップセット初期化
	strcpy(type_3, "");										// データタイプ初期化
	origin_x_3     = 0;										// 地図原点x座標初期化
	origin_y_3     = 0;										// 地図原点y座標初期化
	size_x_3       = 0;										// 地図範囲x座標初期化
	size_y_3       = 0;										// 地図範囲y座標初期化
	strcpy(explanation_3, "");									// 地図説明初期化
	map_origin_x_3 = 0;										// 地図原点x座標初期化
	map_origin_y_3 = 0;										// 地図原点y座標初期化
	map_size_x_3   = 0;										// 地図サイズx座標初期化
	map_size_y_3   = 0;										// 地図サイズy座標初期化
	pixel_size_x   = 0;										// ピクセルサイズx座標初期化
	pixel_size_y   = 0;										// ピクセルサイズy座標初期化

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーション読み込み] ---------------------------------------------------//

	strcpy(location , m_01_location.c_str());							// データ格納 [地域名]
	strcpy(mapset   , m_02_mapset.c_str());								// データ格納 [利用者マップセット名]
	strcpy(type     , m_03_type.c_str());								// データ格納 [データタイプ]
	origin_x     = m_04_originx;									// データ格納 [対象原点x座標]
	origin_y     = m_05_originy;									// データ格納 [対象原点y座標]
	size_x       = m_06_sizex;									// データ格納 [対象範囲x座標]
	size_y       = m_07_sizey;									// データ格納 [対象範囲y座標]
	pixel_size_x = m_08_pixelsizex;									// データ格納 [ピクセルサイズx座標]
	pixel_size_y = m_09_pixelsizey;									// データ格納 [ピクセルサイズy座標]
	strcpy(gui,       m_10_gui.c_str());								// データ格納 [GUI動作]
	strcpy(gui_addr , m_11_guiaddress.c_str());							// データ格納 [GUIプログラムアドレス]
	strcpy(ip,        m_12_ip.c_str());								// データ格納 [IPアドレス]
	strcpy(send_port, m_13_sendport.c_str());							// データ格納 [送信ポート番号]
	strcpy(recv_port, m_14_receiveport.c_str());							// データ格納 [受信ポート番号]

	//---------------------------------------------------------------------------------------//


	//-- [デバッグconfiguration] ------------------------------------------------------------//

	#ifndef DEBUG_CONFIGURATION									// デバッグ表示開始

	printf("\n[configuration]\n");									// label
	printf("01_location    = %s\n", location);							// location
	printf("02_mapset      = %s\n", mapset);							// mapset
	printf("03_type        = %s\n", type);								// type
	printf("04_originx     = %ld\n", origin_x);							// origin_x
	printf("05_originy     = %ld\n", origin_y);							// origin_y
	printf("06_sizex       = %ld\n", size_x);							// size_x
	printf("07_sizey       = %ld\n", size_y);							// size_y
	printf("08_pixelsizex  = %ld\n", pixel_size_x);							// pixel_size_x
	printf("09_pixelsizey  = %ld\n", pixel_size_y);							// pixel_size_y
	printf("10_gui         = %s\n", gui);								// gui
	printf("11_guiaddress  = %s\n", gui_addr);							// guiaddress
	printf("12_ip          = %s\n", ip);								// ip
	printf("13_sendport    = %s\n", send_port);							// sendport
	printf("14_receiveport = %s\n\n", recv_port);							// receiveport
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーションチェック] ---------------------------------------------------//

	// GUI動作チェック
	if(strcmp("off", gui) == 0) gui_flag = 0;							// GUI機能OFF
	else if(strcmp("on", gui) == 0) gui_flag = 1;							// GUI機能ON
	else{

		printf("configuration error:10_gui = %s\n", gui);					// エラーメッセージ
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		return RTC::RTC_ERROR;									// 戻り値 [異常終了]

	}

	//---------------------------------------------------------------------------------------//


	//-- [データ格納] -----------------------------------------------------------------------//

	m_MeshOrder.mode         = 0;									// アクセスモード格納

	for(i = 0; i < DATASIZE; i++){

		m_MeshOrder.location[i] = location[i];							// 地域名格納
		if(location[i] == 0x00) break;								// 終端文字チェック

	}

	for(i = 0; i < DATASIZE; i++){

		m_MeshOrder.mapset[i] = mapset[i];							// 利用者マップセット格納
		if(mapset[i] == 0x00) break;								// 終端文字チェック

	}

	for(i = 0; i < DATASIZE; i++){

		m_MeshOrder.type[i] = type[i];								// データタイプ格納
		if(type[i] == 0x00) break;								// 終端文字チェック

	}

	m_MeshOrder.origin_x     = origin_x;								// 対象原点x座標格納
	m_MeshOrder.origin_y     = origin_y;								// 対象原点y座標格納
	m_MeshOrder.size_x       = size_x;								// 対象範囲x座標格納
	m_MeshOrder.size_y       = size_y;								// 対象範囲y座標格納
	m_MeshOrder.pixel_size_x = pixel_size_x;							// ピクセルサイズx座標格納
	m_MeshOrder.pixel_size_y = pixel_size_y;							// ピクセルサイズy座標格納

	//---------------------------------------------------------------------------------------//


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [GUIプログラム起動] --------------------------------------------------------//

		strcpy(command, gui_addr);								// 実行プログラム
		strcat(command, " ");									// 空白
		strcat(command, ip);									// オプション [IPアドレス]
		strcat(command, " ");									// 空白
		strcat(command, recv_port);								// オプション [受信ポート番号]
		strcat(command, " ");									// 空白
		strcat(command, send_port);								// オプション [送信ポート番号]
		strcat(command, " &");									// オプション [バックグランド]
		system(command);									// GUIプログラム起動
		usleep(WAITTIME);									// 待機 WAITTIME [us]

		//-------------------------------------------------------------------------------//


		//-- [UDP通信関連初期化] --------------------------------------------------------//

		memset(send_data, 0, sizeof(send_data));						// 送信データ初期化
		memset(recv_data, 0, sizeof(recv_data));						// 受信データ初期化

		//-------------------------------------------------------------------------------//


		//-- [ソケット作成] -------------------------------------------------------------//
		//-										-//
		//- プロトコル・ファミリ							-//
		//- AF_UNIX, AF_LOCAL	: ローカル通信						-//
		//- AF_INET		: IPv4 インターネットプロトコル				-//
		//- AF_INET6		: IPv6 インターネットプロトコル				-//
		//- AF_IPX		: IPX-NoveIIプロトコル					-//
		//- AF_NETLINK		: カーネルユーザデバイス				-//
		//- AF_X25		: ITU-T X.25/ISO-8208プロトコル				-//
		//- AF_AX25		: アマチュア無線AX.25プロトコル				-//
		//- AF_ATMPVC		: ATMPVC						-//
		//- AF_APPLETALK	: アップルトーク					-//
		//- AF_PACKET		: 低レベルパケットインターフェイス			-//
		//-										-//
		//- 通信方式									-//
		//- SOCK_STREAM		: バイトストリーム	[信頼性有:順序保証有:双方向]	-//
		//- SOCK_DGRAM		: データグラム		[信頼性無:固定最大長]		-//
		//- SOCK_SEQPACKET	: データグラム		[信頼性有:固定最大長:双方向]	-//
		//- SOCK_RAW		: 生ネットワーク					-//
		//- SOCK_RDW		: データグラム		[信頼性有:順序保証無]		-//
		//- SOCK_PACKET		: 廃止							-//
		//-										-//
		//-------------------------------------------------------------------------------//

		if((send_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){					// ソケット作成 [送信データ用]

			printf("socket error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		if((recv_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){					// ソケット作成 [受信データ用]

			printf("socket error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [アドレス設定] -------------------------------------------------------------//

		send_addr.sin_family      = AF_INET;							// アドレスファミリ
		send_addr.sin_port        = htons(atoi(send_port));					// ポート番号 [バイトオーダ]
		send_addr.sin_addr.s_addr = inet_addr(ip);						// IPアドレス [バイトオーダ]

		recv_addr.sin_family      = AF_INET;							// アドレスファミリ
		recv_addr.sin_port        = htons(atoi(recv_port));					// ポート番号 [バイトオーダ]
		recv_addr.sin_addr.s_addr = inet_addr(ip);						// IPアドレス [バイトオーダ]

		//-------------------------------------------------------------------------------//


		//-- [受信設定] -----------------------------------------------------------------//

		bind(recv_sock, (struct sockaddr *)&recv_addr, sizeof(recv_addr));			// バインド [受信データ]
		addr_size = sizeof(recv_addr);								// アドレス格納構造体サイズ格納

		//-------------------------------------------------------------------------------//


		//-- [シグナルハンドル設定]  ----------------------------------------------------//

		saio.sa_handler = signal_handler_IO;							// シグナルハンドル
		sigemptyset(&saio.sa_mask);								// 追加マスクシグナル
		saio.sa_flags = 0;									// 動作詳細設定フラグ
		saio.sa_restorer = NULL;								// オボソリート [= NULL]

		//-------------------------------------------------------------------------------//


		//-- [シグナルハンドル調査/変更] ------------------------------------------------//

		sigaction(SIGIO, &saio, NULL);								// シグナルアクション調査/変更

		//-------------------------------------------------------------------------------//


		//-- [非同期通信モード設定] -----------------------------------------------------//

		fcntl(recv_sock, F_SETOWN, getpid());							// シグナル受け取り設定 [プロセス]
		fcntl(recv_sock, F_SETFL, O_NONBLOCK | FASYNC);						// 非同期通信モード設定

		//-------------------------------------------------------------------------------//


		//-- [送信データセット] ---------------------------------------------------------//

		strcpy(send_data, "send");								// 送信データ設定

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t Dijkstra::onDeactivated(RTC::UniqueId ec_id)
{


	//-- [GUI 関連] -------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [GUIプログラム終了コマンド送信] --------------------------------------------//

		strcpy(send_data, "quit");								// 送信データ設定

		if(sendto(send_sock, send_data, strlen(send_data), 0,
			(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){				// データ送信

			printf("send error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [ソケットディスクリプタクローズ] -------------------------------------------//

		close(send_sock);									// ソケットディスクリプタクローズ
		close(recv_sock);									// ソケットディスクリプタクローズ

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t Dijkstra::onExecute(RTC::UniqueId ec_id)
{


	//-- [データ更新] -----------------------------------------------------------------------//

	if(m_RobotPositionIn.isNew()){									// 更新チェック

		m_RobotPositionIn.read();								// データ読み込み


		//-- [データ格納] ---------------------------------------------------------------//

		sec         = m_RobotPosition.tm.sec;							// タイムスタンプ格納 [sec]
		nsec        = m_RobotPosition.tm.nsec;							// タイムスタンプ格納 [nsec]
		robot_x     = (long int)(m_RobotPosition.x * 1000.0);					// ロボットx座標格納 [mm]
		robot_y     = (long int)(m_RobotPosition.y * 1000.0);					// ロボットy座標格納 [mm]
		robot_theta = m_RobotPosition.theta;							// ロボットtheta格納 [rad]

		//-------------------------------------------------------------------------------//


		//-- [データチェック] -----------------------------------------------------------//

		// 範囲チェック [x座標]
		if((robot_x < origin_x) || ((origin_x + (long)size_x) < robot_x)){

			printf("error [robot_x:outside range]\n");					// エラーメッセージ
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		// 範囲チェック [y座標]
		if((robot_y < origin_y) || ((origin_y + (long)size_y) < robot_y)){

			printf("error [robot_y:outside range]\n");					// エラーメッセージ
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [デバッグinport] -----------------------------------------------------------//

		#ifndef DEBUG_INPORT									// デバッグ表示開始

		printf("\n[inport - robot]\n");								// label
		printf("timestamp = %ld.%ld\n", sec , nsec);						// timestamp
		printf("robot_x   = %lf\n", robot_x / 1000.0);						// robot_x
		printf("robot_y   = %lf\n\n", robot_y / 1000.0);					// robot_y
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


	}

	if(m_GoalPositionIn.isNew()){									// 更新チェック

		m_GoalPositionIn.read();								// データ読み込み


		//-- [データ格納] ---------------------------------------------------------------//

		sec_2  = m_GoalPosition.tm.sec;								// タイムスタンプ格納 [sec]
		nsec_2 = m_GoalPosition.tm.nsec;							// タイムスタンプ格納 [nsec]
		goal_x = (long int)(m_GoalPosition.x * 1000.0);						// ゴールx座標格納 [mm]
		goal_y = (long int)(m_GoalPosition.y * 1000.0);						// ゴールy座標格納 [mm]

		//-------------------------------------------------------------------------------//


		//-- [データチェック] -----------------------------------------------------------//

		// 範囲チェック [x座標]
		if((goal_x < origin_x) || ((origin_x + (long)size_x) < goal_x)){

			printf("error [goal_x:outside range]\n");					// エラーメッセージ
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		// 範囲チェック [y座標]
		if((goal_y < origin_y) || ((origin_y + (long)size_y) < goal_y)){

			printf("error [goal_y:outside range]\n");					// エラーメッセージ
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [デバッグinport] -----------------------------------------------------------//

		#ifndef DEBUG_INPORT									// デバッグ表示開始

		printf("\n[inport - goal]\n");								// label
		printf("timestamp = %ld.%ld\n", sec_2 , nsec_2);					// timestamp
		printf("goal_x   = %lf\n", goal_x / 1000.0);						// goal_x
		printf("goal_y   = %lf\n\n", goal_y / 1000.0);						// goal_y
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [データ格納] ---------------------------------------------------------------//

		m_MeshOrder.tm.sec  = sec_2;								// タイムスタンプ格納 [sec]
		m_MeshOrder.tm.nsec = nsec_2;								// タイムスタンプ格納 [nsec]
		m_MeshOrder.trigger = 0;								// 動作トリガー格納

		//-------------------------------------------------------------------------------//


		//-- [デバッグoutport] ----------------------------------------------------------//

		#ifndef DEBUG_OUTPORT									// デバッグ表示開始

		printf("\n[outport]\n");								// label
		printf("timestamp    = %ld,%ld\n", sec_2, nsec_2);					// timestamp
		printf("trigger      = 0\n");								// trigger
		printf("mode         = 0\n");								// mode
		printf("location     = %s\n", location);						// location
		printf("mapset       = %s\n", mapset);							// mapset
		printf("type         = %s\n", type);							// type
		printf("origin_x     = %ld\n", origin_x);						// origin_x
		printf("origin_y     = %ld\n", origin_y);						// origin_y
		printf("size_x       = %ld\n", size_x);							// size_x
		printf("size_y       = %ld\n", size_y);							// size_y
		printf("pixel_size_x = %ld\n", pixel_size_x);						// pixel_size_x
		printf("pixel_size_y = %ld\n\n", pixel_size_y);						// pixel_size_y
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [データ出力] ---------------------------------------------------------------//

		m_MeshOrderOut.write();									// データ出力

		//-------------------------------------------------------------------------------//


		//-- [データ格納] ---------------------------------------------------------------//

		m_MeshOrder.tm.sec  = sec_2;								// タイムスタンプ格納 [sec]
		m_MeshOrder.tm.nsec = nsec_2;								// タイムスタンプ格納 [nsec]
		m_MeshOrder.trigger = 1;								// 動作トリガー格納

		//-------------------------------------------------------------------------------//


		//-- [デバッグoutport] ----------------------------------------------------------//

		#ifndef DEBUG_OUTPORT									// デバッグ表示開始

		printf("\n[outport]\n");								// label
		printf("timestamp    = %ld,%ld\n", sec_2, nsec_2);					// timestamp
		printf("trigger      = 1\n");								// trigger
		printf("mode         = 0\n");								// mode
		printf("location     = %s\n", location);						// location
		printf("mapset       = %s\n", mapset);							// mapset
		printf("type         = %s\n", type);							// type
		printf("origin_x     = %ld\n", origin_x);						// origin_x
		printf("origin_y     = %ld\n", origin_y);						// origin_y
		printf("size_x       = %ld\n", size_x);							// size_x
		printf("size_y       = %ld\n", size_y);							// size_y
		printf("pixel_size_x = %ld\n", pixel_size_x);						// pixel_size_x
		printf("pixel_size_y = %ld\n\n", pixel_size_y);						// pixel_size_y
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [データ出力] ---------------------------------------------------------------//

		m_MeshOrderOut.write();									// データ出力

		//-------------------------------------------------------------------------------//


	}else if(m_MeshDataIn.isNew()){									// 更新チェック

		m_MeshDataIn.read();									// データ読み込み


		//-- [データ格納] ---------------------------------------------------------------//

		sec_3          = m_MeshData.tm.sec;							// タイムスタンプ格納 [sec]
		nsec_3         = m_MeshData.tm.nsec;							// タイムスタンプ格納 [nsec]
		mode_3         = m_MeshData.mode;							// アクセスモード格納

		for(i = 0; i < DATASIZE; i++){

			location_3[i] = m_MeshData.location[i];						// 地域名格納
			if(m_MeshData.location[i] == 0x00) break;					// 終端文字チェック

		}

		for(i = 0; i < DATASIZE; i++){

			mapset_3[i] = m_MeshData.mapset[i];						// 利用者マップセット格納
			if(m_MeshData.mapset[i] == 0x00) break;						// 終端文字チェック

		}

		for(i = 0; i < DATASIZE; i++){

			type_3[i] = m_MeshData.type[i];							// データタイプ格納
			if(m_MeshData.type[i] == 0x00) break;						// 終端文字チェック

		}

		origin_x_3     = m_MeshData.origin_x;							// 対象原点x座標格納
		origin_y_3     = m_MeshData.origin_y;							// 対象原点y座標格納
		size_x_3       = m_MeshData.size_x;							// 対象範囲x座標格納
		size_y_3       = m_MeshData.size_y;							// 対象範囲y座標格納

		for(i = 0; i < DATASIZE; i++){

			explanation_3[i] = m_MeshData.explanation[i];					// 地図説明格納
			if(m_MeshData.explanation[i] == 0x00) break;					// 終端文字チェック

		}

		map_origin_x_3 = m_MeshData.map_origin_x;						// 地図原点x座標格納
		map_origin_y_3 = m_MeshData.map_origin_y;						// 地図原点y座標格納
		map_size_x_3   = m_MeshData.map_size_x;							// 地図サイズx座標格納
		map_size_y_3   = m_MeshData.map_size_y;							// 地図サイズy座標格納
		pixel_size_x_3 = m_MeshData.pixel_size_x;						// ピクセルサイズx座標格納
		pixel_size_y_3 = m_MeshData.pixel_size_y;						// ピクセルサイズy座標格納

		for(i = 0; i < DATASIZE; i++){

			category_3[i] = m_MeshData.category[i];						// カテゴリデータ格納
			if(m_MeshData.category[i] == 0x00) break;					// 終端文字チェック

		}


		//-- [メモリ確保] ---------------------------------------------------------------//

		x = size_x_3 / pixel_size_x_3;								// x座標個数確保
		y = size_y_3 / pixel_size_y_3;								// y座標個数確保

		pmesh = (unsigned long*) calloc(x * y, sizeof(unsigned long));				// メモリ確保

		if(pmesh == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < (unsigned long)(x * y); i++) *(pmesh + i) = m_MeshData.mapdata[i];	// 地図データ格納

		//-------------------------------------------------------------------------------//


		//-------------------------------------------------------------------------------//


		//-- [デバッグmeshdata] ---------------------------------------------------------//

		#ifndef DEBUG_MESHDATA									// デバッグ表示開始

		printf("\n[meshdata]\n");								// label
		printf("timestamp    = %ld.%ld\n", sec_3, nsec_3);					// timestamp_3
		printf("mode         = %d\n", mode_3);							// mode_3
		printf("location     = %s\n", location_3);						// location_3
		printf("mapset       = %s\n", mapset_3);						// mapset_3
		printf("type         = %s\n", type_3);							// type_3
		printf("origin_x     = %ld\n", origin_x_3);						// origin_x_3
		printf("origin_y     = %ld\n", origin_y_3);						// origin_y_3
		printf("size_x       = %ld\n", size_x_3);						// size_x_3
		printf("size_y       = %ld\n", size_y_3);						// size_y_3
		printf("explanation  = %s\n", explanation_3);						// explanation_3
		printf("map_origin_x = %ld\n", map_origin_x_3);						// map_origin_x_3
		printf("map_origin_y = %ld\n", map_origin_y_3);						// map_origin_y_3
		printf("map_size_x   = %ld\n", map_size_x_3);						// map_size_x_3
		printf("map_size_y   = %ld\n", map_size_y_3);						// map_size_y_3
		printf("pixel_size_x = %ld\n", pixel_size_x_3);						// pixel_size_x_3
		printf("pixel_size_y = %ld\n", pixel_size_y_3);						// pixel_size_y_3
		printf("category     = %s\n", category_3);						// mapdata
		printf("mapdata      =\n");

		for(i = 0; i < y; i++){

			for(j = 0; j < x; j++) printf("%ld", *(pmesh + x * i + j));			// データ表示
			printf("\n");									// 改行表示

		}

		printf("\n");										// 改行表示

		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [カテゴリ処理] -------------------------------------------------------------//

		j = 0;											// カウント初期化
		category_num = 0;									// カテゴリ番号初期化
		category_flag = 0;									// フラグクリア

		for(i = 0; i < DATASIZE; i++){

			if((category_3[i] == ',') || (category_3[i] == 0x00)){				// 区切りチェック

				category_buf[category_num][j] = 0x00;					// 終端文字格納
				j = 0;									// カウント初期化

				if(category_num == 1){

					if(strcmp("hazard", category_buf[1]) == 0){			// カテゴリチェック

						hazard = atol(category_buf[0]);				// ハザード番号格納
						if(category_flag == 2) break;				// カテゴリ処理終了
						category_flag = 1;					// フラグチェック

					}else if(strcmp("safety", category_buf[1]) == 0){		// カテゴリチェック

						safety = atol(category_buf[0]);				// セーフティ番号格納
						if(category_flag == 1) break;				// カテゴリ処理終了
						category_flag = 2;					// フラグチェック

					}

					category_num = 0;						// カウント変更

				}else{

					category_num = 1;						// カウント変更

				}

				if(category_3[i] == 0x00){						// 終端文字チェック

					printf("category error!! [hazard]\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

			}else{

				category_buf[category_num][j] = category_3[i];				// データ格納
				j++;									// カウント

			}

		}

		//-------------------------------------------------------------------------------//


		//-- [入力処理] -----------------------------------------------------------------//

		start_x = (unsigned long)((robot_x - origin_x_3) / pixel_size_x_3);			// START地点x座標格納
		start_y = (unsigned long)(y - (robot_y - origin_y_3) / pixel_size_y_3 - 1);		// START地点y座標格納
		end_x   = (unsigned long)((goal_x  - origin_x_3) / pixel_size_x_3);			// END地点x座標格納
		end_y   = (unsigned long)(y - (goal_y  - origin_y_3) / pixel_size_y_3 - 1);		// END地点y座標格納

		if(*(pmesh + x * start_y + start_x) == hazard){

			printf("robot position error!! [hazard]\n");					// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		if(*(pmesh + x * end_y + end_x) == hazard){

			printf("goal position error!! [hazard]\n");					// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [メモリ確保] ---------------------------------------------------------------//

		pd = (double *) calloc(x * y * x * y, sizeof(double));					// メモリ確保

		if(pd == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < (x * y); i++){

			for(j = 0; j < (x * y); j++) *(pd + i * x * y + j) = 9999.0;			// リンク長初期化

		}

		//-------------------------------------------------------------------------------//


		//-- [リンク長処理] -------------------------------------------------------------//
		//- 位置関係									-//
		//- [(i * x * y) + i - x - 1] [(i * x * y) + i - x] [(i * x * y) + i - x + 1]	-//
		//- [(i * x * y) + i     - 1] [(i * x * y) + i    ] [(i * x * y) + i     + 1]	-//
		//- [(i * x * y) + i + x - 1] [(i * x * y) + i + x] [(i * x * y) + i + x + 1]	-//
		//-------------------------------------------------------------------------------//

		for(i = 0; i < x * y; i++){

			if(*(pmesh + i) == safety){						// 自身 -> 安全地帯

				if((i >= x) && ((i % x) != 0))					// 左上チェック
					if(*(pmesh + i - x - 1) == safety)
						*(pd + i * x * y + i - x - 1) = 1.414;		// 左上 -> 安全地帯

				if(i >= x)							// 上チェック
					if(*(pmesh + i  - x   ) == safety)
						*(pd + i * x * y + i - x    ) = 1.000;		// 上   -> 安全地帯

				if((i >= x) && ((i % x) != (x - 1)))				// 右上チェック
					if(*(pmesh + i - x + 1) == safety)
						*(pd + i * x * y + i - x + 1) = 1.414;		// 右上 -> 安全地帯

				if((i % x) != 0)						// 左チェック
					if(*(pmesh + i     - 1) == safety)
						*(pd + i * x * y + i     - 1) = 1.000;		// 左   -> 安全地帯

				if((i % x) != (x - 1))						// 右チェック
					if(*(pmesh + i     + 1) == safety)
						*(pd + i * x * y + i     + 1) = 1.000;		// 右   -> 安全地帯

				if((i < x * (y - 1)) && ((i % x) != 0))				// 左下チェック
					if(*(pmesh + i + x - 1) == safety)
						*(pd + i * x * y + i + x - 1) = 1.414;		// 左下 -> 安全地帯

				if(i < x * (y - 1))						// 下チェック
					if(*(pmesh + i + x    ) == safety)
						*(pd + i * x * y + i + x    ) = 1.000;		// 左下 -> 安全地帯

				if((i < x * (y - 1)) && ((i % x) != (x-1)))			// 右下チェック
					if(*(pmesh + i + x + 1) == safety)
						*(pd + i * x * y + i + x + 1) = 1.414;		// 左下 -> 安全地帯

			}

		}

		//-------------------------------------------------------------------------------//


		//-- [デバッグlink] -------------------------------------------------------------//

		#ifndef DEBUG_LINK									// デバッグ表示開始

		fd = fopen("test.txt", "w");								// ファイルオープン

		if(fd == NULL){

			printf("file open error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < y; i++){

			for(j = 0; j < x; j++) fprintf(fd, "%ld", *(pmesh + x * i + j));		// データ表示
			fprintf(fd, "\n");								// 改行表示

		}

		for(i = 0; i < x * y; i++){

			fprintf(fd, "\n[%ld - %ld]\n", i / x, i % x);					// データ書き込み

			for(j = 0; j < x * y; j++){

				if(*(pd + i * x * y + j) == 9999.0) fprintf(fd, "_");			// データ書き込み
				else fprintf(fd, "@");							// データ書き込み
				if(((j + 1) % x) == 0) fprintf(fd, "\n");				// 改行書き込み

			}

			fprintf(fd, "\n");								// 改行書き込み

		}

		fclose(fd);										// ファイルクローズ

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [要素処理] -----------------------------------------------------------------//

		pv = (double *) calloc(x * y, sizeof(double));						// メモリ確保

		if(pv == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		pp = (long int *) calloc(x * y, sizeof(long int));					// メモリ確保

		if(pp == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < (x * y); i++){

			*(pv + i) = 9999.0;								// 最短距離初期化
			*(pp + i) = 0;									// 直前ノード初期化

		}

		*(pv + x * start_y + start_x) = 0.0;							// START地点最短距離初期化

		pm = (long int *) calloc(x * y, sizeof(long int));					// メモリ確保

		if(pm == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		nm = x * y - 1;										// 未求ノード要素数初期化

		for(i = 0; i < nm; i++){

			if(i < x * start_y + start_x) *(pm + i) = i;					// 未求ノード初期化
			else *(pm + i) = i + 1;								// 未求ノード初期化

		}

		prpath = (long int *) calloc(x * y, sizeof(long int));					// メモリ確保

		if(prpath == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [デバッグinitialization] ---------------------------------------------------//

		#ifndef DEBUG_INITIALIZATION								// デバッグ表示開始

		fd = fopen("test2.txt", "w");								// ファイルオープン

		if(fd == NULL){

			fprintf(fd, "file open error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		fprintf(fd, "\n[pv]\n");								// データ書き込み

		for(i = 0; i < (x * y); i++){

			if(*(pv + i) == 9999.0) fprintf(fd, "9 ");					// データ書き込み
			else fprintf(fd, "@ ");								// データ書き込み
			if(((i + 1) % x) == 0) fprintf(fd, "\n");					// 改行書き込み

		}

		fprintf(fd, "\n[pp]\n");								// データ書き込み

		for(i = 0; i < (x * y); i++){

			fprintf(fd, "%ld ", *(pp + i));							// データ書き込み
			if(((i + 1) % x) == 0) fprintf(fd, "\n");					// 改行書き込み

		}

		fprintf(fd, "\n[nm]\n");								// データ書き込み

		for(i = 0; i < nm; i++){

			fprintf(fd, "%ld ", *(pm + i));							// データ書き込み
			if(((i + 1) % x) == 0) fprintf(fd, "\n");					// 改行書き込み

		}

		fclose(fd);										// ファイルクローズ

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [ダイクストラ法] -----------------------------------------------------------//

		nstep = 1;										// 反復計算回数初期化
		is = x * start_y + start_x;								// ノード初期化


		//-- [反復計算] -----------------------------------------------------------------//

		while(nm != 0){

			nstep++;									// カウント

			for(i = 0; i < nm; i++){

				j = *(pm + i);								// 未求ノードセット

				if((*(pd + is * x * y + j) < 9999.0) &&
					(*(pv + is) + *(pd + is * x * y + j)) < *(pv + j)){

					*(pv + j) = *(pv + is) + *(pd + is * x * y + j);		// 最短距離データ格納
					*(pp + j) = is;							// 直前ノードデータ格納

				}

			}

			vmin = 10000000.0;								// 最短距離初期化

			for(i = 0; i < nm; i++){

				j = *(pm + i);								// 未求ノードセット

				if(*(pv + j) < vmin){

					vmin = *(pv + j);						// データ置き換え
					is = j;								// データ置き換え

				}

			}

			nm--;										// カウントダウン

			for(i = 0; i < nm; i++){

				if(*(pm + i) >= is) *(pm + i) = *(pm + i + 1);				// 比較データ置き換え

			}

		}

		//-------------------------------------------------------------------------------//


		//-- [経路データ生成] -----------------------------------------------------------//

		i = x * end_y + end_x;									// END地点初期化
		j = 0;											// カウント初期化

		if(i != x * start_y + start_x){

			ipn = i;									// データ置き換え

			while(ipn != x * start_y + start_x){

				*(prpath + j) = *(pp + ipn);						// 逆ルートデータ格納
				j++;									// カウント
				ipn = *(pp + ipn);							// ポイント切り替え

			}

		}

		//-------------------------------------------------------------------------------//


		//-- [デバッグRPATH] ---------------------------------------------------//

		#ifndef DEBUG_RPATH									// デバッグ表示開始

		printf("\n[rpath]\n");									// label

		for(i = 0; i < j; i++){

			printf("[%ld - %ld]\n", *(prpath + i) % x, *(prpath + i) / x);			// データ表示

		}

		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [メモリ解放] ---------------------------------------------------------------//

		free(pd);										// メモリ解放
		free(pv);										// メモリ解放
		free(pp);										// メモリ解放
		free(pm);										// メモリ解放

		//-------------------------------------------------------------------------------//


		//-------------------------------------------------------------------------------//


		//-- [GUI関連] ------------------------------------------------------------------//

		if(gui_flag == 1){									// GUI機能ON


			//-- [地図データ処理] ---------------------------------------------------//

			*(pmesh + x * start_y + start_x) = STARTENDNO;					// START地点追記
			*(pmesh + x * end_y   + end_x  ) = STARTENDNO;					// END地点追記

			for(i = 0; i < (j - 1); i++){

				*(pmesh + x * (*(prpath + i) / x) + (*(prpath + i) % x)) = PATHNO;	// PATH地点追記

			}


			//-- [デバッグmeshpathdata] ---------------------------------------------//

			#ifndef DEBUG_MESHPATHDATA							// デバッグ表示開始

			printf("\n[meshpathdata]\n");							// label

			for(i = 0; i < y; i++){

				for(j = 0; j < x; j++) printf("%ld", *(pmesh + x * i + j));		// データ表示
				printf("\n");								// 改行表示

			}

			printf("\n");									// 改行表示

			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			//-----------------------------------------------------------------------//


			//-- [データ送信] -------------------------------------------------------//

			sprintf(send_data, "%ld,%ld,%d,%s,%s,%s,%ld,%ld,%ld,%ld,%s,%ld,%ld,%ld,%ld,%ld,%ld,%ld",
				sec_3, nsec_3, mode_3, location_3, mapset_3, type_3, origin_x_3, origin_y_3,
				size_x_3, size_y_3, explanation_3, map_origin_x_3, map_origin_y_3,
				map_size_x_3, map_size_y_3, pixel_size_x_3, pixel_size_y_3, x * y + 1);	// データ格納

			if(sendto(send_sock, send_data, strlen(send_data), 0,
				(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){			// データ送信

				printf("send error!!\n");						// メッセージ表示
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				return RTC::RTC_ERROR;							// 戻り値 [異常終了]

			}

			psend = (char*)calloc(x * y + 1, sizeof(char));					// メモリ確保
			for(i = 0; i < x * y; i++) *(psend + i) = *(pmesh + i) + 0x30;			// データ格納
			*(psend + i) = 0x00;								// 終端文字格納

			for(i = 0; i < y; i++){

				if(sendto(send_sock, psend + x * i, x, 0,
					(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){		// データ送信

					printf("send error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				usleep(1000);								// 待機

			}

			free(psend);									// メモリ解放

			//-----------------------------------------------------------------------//


		}

		//-------------------------------------------------------------------------------//


		//-- [メモリ解放] ---------------------------------------------------------------//

		path_buf[0] = *(pmesh + x * start_y + start_x);						// START地点格納
		free(pmesh);										// メモリ解放

		//-------------------------------------------------------------------------------//


		//-- [経路処理] -----------------------------------------------------------------//

		ppath = (long int *) calloc(j, sizeof(long int));					// メモリ確保

		if(ppath == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < j - 1; i++){

			*(ppath + i) = *(prpath + j - i - 2);

		}

		*(ppath + i) = x * end_y + end_x;							// END地点追記


		//-- [デバッグPATH] -------------------------------------------------------------//

		#ifndef DEBUG_PATH									// デバッグ表示開始

		printf("\n[path]\n");									// label

		for(i = 0; i < j; i++){

			printf("[%ld - %ld]\n", *(ppath + i) % x, y - (*(ppath + i) / x) - 1);		// データ表示

		}

		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [経路正規化] ---------------------------------------------------------------//

		ppath_new = (long int *) calloc(j, sizeof(long int));					// メモリ確保

		if(ppath_new == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		path_num = 0;										// 経路番号初期化
		path_buf[1] = *ppath;									// 経路予測格納

		for(i = 0; i < j; i++){

			if(*(ppath + i) != path_buf[1]){

				*(ppath_new + path_num) = path_buf[0];					// 経路格納
				path_num++;								// カウント

			}

			path_buf[1] = *(ppath + i) + (*(ppath + i) - path_buf[0]);			// 経路予測格納
			path_buf[0] = *(ppath + i);							// 前経路格納

		}

		*(ppath_new + path_num) = *(ppath + i - 1);						// END地点格納

		//-------------------------------------------------------------------------------//


		//-- [デバッグPATHNEW] ----------------------------------------------------------//

		#ifndef DEBUG_PATHNEW									// デバッグ表示開始

		printf("\n[path_new]\n");								// label

		for(i = 0; i <= path_num; i++){

			printf("[%ld - %ld]\n", *(ppath_new + i) % x, y - (*(ppath_new + i) / x) - 1);	// データ表示

		}

		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-------------------------------------------------------------------------------//


		//-- [データ出力] ---------------------------------------------------------------//

		m_Path.pose.length(path_num + 2);							// メモリ確保 [pose]

		m_Path.tm.sec  = sec_3;									// タイムスタンプ格納 [sec]
		m_Path.tm.nsec = nsec_3;								// タイムスタンプ格納 [nsec]

		m_Path.pose[0].position.x = (double)(robot_x) / 1000.0;					// ルートx座標格納 [robot]
		m_Path.pose[0].position.y = (double)(robot_y) / 1000.0;					// ルートx座標格納 [robot]
		m_Path.pose[0].heading    = robot_theta;						// ルートheading格納 [robot]

		for(i = 0; i < path_num; i++){

			m_Path.pose[i + 1].position.x = (double)((long)((*(ppath_new + i) % x) * pixel_size_x_3
				+ (pixel_size_x_3 / 2)) + origin_x_3) / 1000.0;				// ルートx座標格納
			m_Path.pose[i + 1].position.y = (double)((long)((y - (*(ppath_new + i) / x) - 1) * pixel_size_y_3
				+ (pixel_size_y_3 / 2)) + origin_y_3) / 1000.0;				// ルートy座標格納
			m_Path.pose[i + 1].heading    = atan2(m_Path.pose[i + 1].position.y - m_Path.pose[i].position.y,
				m_Path.pose[i + 1].position.x - m_Path.pose[i].position.x);		// ルートheading格納
			if(m_Path.pose[i + 1].heading < 0) m_Path.pose[i + 1].heading += (2.0 * PI);	// 負処理

		}

		m_Path.pose[i + 1].position.x = (double)(goal_x) / 1000.0;				// ルートx座標格納 [goal]
		m_Path.pose[i + 1].position.y = (double)(goal_y) / 1000.0;				// ルートx座標格納 [goal]
		m_Path.pose[i + 1].heading    = atan2(m_Path.pose[i + 1].position.y - m_Path.pose[i].position.y,
			m_Path.pose[i + 1].position.x - m_Path.pose[i].position.x);			// ルートheading格納 [goal]
		if(m_Path.pose[i + 1].heading < 0) m_Path.pose[i + 1].heading += (2.0 * PI);		// 負処理

		m_PathOut.write();									// データ出力

		//-------------------------------------------------------------------------------//


		//-- [メモリ解放] ---------------------------------------------------------------//

		free(prpath);										// メモリ解放
		free(ppath);										// メモリ解放
		free(ppath_new);									// メモリ解放
		m_Path.pose.length(0);									// メモリ解放 [pose]

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [データ受信] ---------------------------------------------------------------//

		if(wait_flag == 0){

			// データ受信
			recv_len = recvfrom(recv_sock, recv_data, sizeof(recv_data), 0,
				(struct sockaddr *)&recv_addr, &addr_size);				// データ受信

			if(recv_len < 0){								// 受信チェック

				printf("receive error!!\n");						// メッセージ表示
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				return RTC::RTC_ERROR;							// 戻り値 [異常終了]

			}


			//-- [デバッグrecv] -----------------------------------------------------//

			#ifndef DEBUG_RECV								// デバッグ表示開始

			printf("\n[recv]\n");								// label
			printf("time      = %lf\n", gettimeofday_sec());				// time
			printf("recv data = %s\n\n", recv_data);					// recv_data
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			memset(recv_data, 0, sizeof(recv_data));					// バッファクリア

			wait_flag = 1;									// フラグセット

		}

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t Dijkstra::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Dijkstra::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Dijkstra::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Dijkstra::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Dijkstra::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void DijkstraInit(RTC::Manager* manager)
  {
    RTC::Properties profile(dijkstra_spec);
    manager->registerFactory(profile,
                             RTC::Create<Dijkstra>,
                             RTC::Delete<Dijkstra>);
  }

};


